package com.gic.finance.service.outer.impl;

import java.util.Date;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.finance.constant.WithdrawalApplyTypeEnum;
import com.gic.finance.constant.WithdrawalStatusEnum;
import com.gic.finance.dto.CashWithdrawalDTO;
import com.gic.finance.entity.TabCashWithdrawal;
import com.gic.finance.entity.TabPayAccount;
import com.gic.finance.qo.CashWithdrawalQueryListQO;
import com.gic.finance.service.CashWithdrawalApiService;
import com.gic.finance.service.CashWithdrawalService;
import com.gic.finance.service.PayAccountService;
import com.gic.store.utils.CreateSerialNumberUtils;

@Service("cashWithdrawalApiService")
public class CashWithdrawalApiServiceImpl implements CashWithdrawalApiService{
    @Autowired
    private CashWithdrawalService cashWithdrawalService;
    @Autowired
    private PayAccountService payAccountService;
    /**
     * 提现申请
     * @Title: cashWithdrawal 
     * @Description:
      * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    private ServiceResponse<Void> cashWithdrawal(CashWithdrawalDTO dto) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(dto, CashWithdrawalDTO.CashWithdrawal.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        dto.setCreateTime(new Date());
        dto.setUpdateTime(new Date());
        dto.setCashWithdrawalStatus(WithdrawalStatusEnum.TO_BE_EXAMINE.getCode());
        //流水号
        dto.setCashWithdrawalSerialNumber(CreateSerialNumberUtils.createSerialNumber());
        cashWithdrawalService.save(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> cashWithdrawalOfServiceProvider(CashWithdrawalDTO dto) {
        dto.setApplyType(WithdrawalApplyTypeEnum.SERVICE_PROVIDER.getCode());
        return cashWithdrawal(dto);
    }

    @Override
    public ServiceResponse<Void> cashWithdrawalOfSupplier(CashWithdrawalDTO dto) {
        dto.setApplyType(WithdrawalApplyTypeEnum.SUPPLIER.getCode());
        return cashWithdrawal(dto);
    }

    @Override
    public ServiceResponse<Void> approval(Integer id) {
        TabCashWithdrawal record = cashWithdrawalService.getById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，无此记录");
        }
        //todo 操作人相关信息

        record.setCashWithdrawalStatus(WithdrawalStatusEnum.PASS.getCode());
        cashWithdrawalService.update(record);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<CashWithdrawalDTO> getDetail(Integer id) {
        TabCashWithdrawal record = cashWithdrawalService.getById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，无此记录");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(CashWithdrawalDTO.class, record));
    }

    @Override
    public ServiceResponse<Void> reject(Integer id, String rejectReason) {
        if (StringUtils.isBlank(rejectReason)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "驳回理由不能为空");
        }
        TabCashWithdrawal record = cashWithdrawalService.getById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，无此记录");
        }
        //todo 操作人相关信息

        record.setCashWithdrawalStatus(WithdrawalStatusEnum.REJECT.getCode());
        record.setRejectReason(rejectReason);
        cashWithdrawalService.update(record);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> pay(Integer id, Integer payId, String bankSerialNumber) {
        if (StringUtils.isBlank(bankSerialNumber)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "银行流水号不能为空");
        }
        TabCashWithdrawal record = cashWithdrawalService.getById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，无此申请单记录");
        }
        TabPayAccount account = payAccountService.getById(payId);
        if (account == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "无此打款户信息记录，请前往添加");
        }
        //todo 操作人相关信息

        record.setPayId(payId);
        record.setPayAccount(account.getBankAccount());
        record.setPayAccountBank(account.getBank());
        record.setPayAccountBranchName(account.getBranchName());
        record.setPayAccountName(account.getAccountName());
        record.setBankSerialNumber(bankSerialNumber);
        record.setCashWithdrawalStatus(WithdrawalStatusEnum.COMPLETE.getCode());
        cashWithdrawalService.update(record);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<CashWithdrawalDTO>> listCashWithdrawalPage(CashWithdrawalQueryListQO params) {
        com.github.pagehelper.Page page = cashWithdrawalService.pageCashWithdrawal(params);
        Page<CashWithdrawalDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, CashWithdrawalDTO.class);
        return ServiceResponse.success(resultPage);
    }

}
